/*
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights reserved.
 *
 * Author: Pops, TCSDeveloper
 */

namespace TopCoder.Rules.DependencyMap.View
{
    using System;
    using System.IO;
    using System.Drawing;
    using System.Drawing.Imaging;

    /// <summary>
    /// The BitmapRenderer implements the the DiagramRenderer and will render diagrams to a BitmapImage.  Please
    /// note that the bitmap image is recreated on every render.  The application should retrieve the image after
    /// the diagram has been rendered to save a reference to the returned diagram.
    /// </summary>
    public class BitMapRenderer
    {
        /// <summary>
        /// This variable represents the pixel format of the image.  The variable is initially set in the
        /// constructor, is mutable via the setter method and will be accessed by the getter method and the
        /// GetGraphics method.
        /// </summary>
        private PixelFormat format;
        
        /// <summary>
        /// This variable represents the <b>last</b> image that was drawn and is recreated each time the GetGraphics
        /// method is called.  This value is set in the GetGraphics method, is mutable and can be null if the
        /// renderer has not been rendered to yet.  This variable is referenced in the getter method.
        /// </summary>
        private Bitmap image = null;
        
        /// <summary>
        /// This property represents the pixel format of the image.
        /// </summary>
        public PixelFormat Format
        {
            get
            { 
                return this.format;
            }
            set
            {
                this.format = value;
            }
        }
        
        /// <summary>
        /// This getter property returns the Image created by this BitmapRenderer.  It will return null if nothing
        /// has been rendered yet.
        /// </summary>
        public Bitmap Image
        {
            get
            {
                return this.image;
            }
        }
        
        /// <summary>
        /// Creates a new instance of the BitmapRenderer class.  This constructor simply saves the passed in
        /// arguments to the class parameters.
        /// </summary>
        public BitMapRenderer(PixelFormat format)
        {
            this.format = format;
        }
        
        /// <summary>
        /// This method will:
        /// 1) Create the Bitmap image using the passed in width and height and the stored pixel format and
        ///    save it to the Image property.
        /// 2) Create and return a GraphicsAdapter using the created Graphics.FromImage(Image) constructor.
        /// </summary>
        /// <exception cref="ArgumentNullException">If depdendency is null.</exception>
        /// <exception cref="ArgumentOutOfRangeException">
        /// If either width or height is less than or equal to zero.
        /// </exception>
        public IGraphics GetGraphics(Dependency dependency, int width, int height)
        {
            if (dependency == null)
            {
                throw new ArgumentNullException("dependency");
            }
            
            if (width <= 0)
            {
                throw new ArgumentOutOfRangeException("width", "Width can not be less than or equal to zero.");
            }

            if (height <= 0)
            {
                throw new ArgumentOutOfRangeException("height", "Height can not be less than or equal to zero.");
            }

            this.image = new Bitmap(width, height, this.format);
            
            return new GraphicsAdapter(Graphics.FromImage(this.image));
        }
    }

}
