/*
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights reserved.
 *
 * Author: Pops, TCSDeveloper
 */

using System.IO;

using System;

namespace TopCoder.Rules.DependencyMap.View
{
    /// <summary>
    /// The SVG Renderer will implement the IDiagramRenderer and will render SVG diagrams to a file.
    /// This renderer can operate in two modes depending on whether the path defines a specific file or directory:
    /// 1) Render to a specific file. If the path specifies a specific file, then the rendered diagram will go
    /// directly to that file. This allows the application to be able to finely specify where the rendering goes to.
    /// 2) Render to a specific directory.  If the path specifies a specific directory, then the rendered diagram
    /// will go into that directory using the dependency's ID as the name of the file. This allows the application
    /// to render multiple diagrams without having to specify the file names for each.
    /// </summary>
    public class SVGRenderer : IDiagramRenderer
    {

        /// <summary>
        /// This variable represents the path (either a file or a directory) that the diagram will be rendered to.
        /// The variable is initially set in the constructor, is mutatable via the setter method and will be
        /// accessed by the getter method and the disposeOf methods.
        /// </summary>
        private string path;

        /// <summary>
        /// This is the property for the path variable. This is a simple setter/getter for the path variable.
        /// </summary>
        /// <exception cref="ArgumentNullException">if the value is null</exception>
        /// <exception cref="ArgumentException">if the value (trim'd) is empty</exception>
        public string Path
        {
            get
            {
                return this.path;
            }
            set
            {
                if (value== null)
                {
                    throw new ArgumentNullException("path");
                }
                if (value.Trim() == string.Empty)
                {
                    throw new ArgumentException("path should not be null.");
                }
                this.path = value;
            }
        }

        /// <summary>
        /// Constructs the SVG Renderer using the specified path. This method simply saves the path to the like
        /// named variable.
        /// </summary>
        /// <param name="path">The file or directory path to render to</param>
        /// <exception cref="ArgumentNullException">if the path is null</exception>
        /// <exception cref="ArgumentException">if the path (trim'd) is empty</exception>
        public SVGRenderer(string path)
        {
            this.Path = path;
        }

        /// <summary>
        /// The method will return a IGraphics object to render with. This method will:
        /// 1) If the path specifies a directory, create the directory if it doesn't exist and then append the
        /// dependency.Title as the file name.
        /// 2) If the path specifies a file, ensure it's directory exists and create the directory structure if not.
        /// 3) Create and return a new SVGGraphicsAdapter(filename, width, height).
        /// </summary>
        /// <param name="dependency">The dependency that will be rendered</param>
        /// <param name="width">The width of the diagram.</param>
        /// <param name="height">The height of the diagram.</param>
        /// <returns>A IGraphics object to render with</returns>
        /// <exception cref="ArgumentNullException">if dependency is null</exception>
        /// <exception cref="ArgumentOutOfRangeException">
        /// If width or height is less than or equal to zero.
        /// </exception>
        public IGraphics GetGraphics(Dependency dependency, int width, int height)
        {
            if (dependency == null)
            {
                throw new ArgumentNullException("dependency");
            }
            
            if (width <= 0)
            {
                throw new ArgumentOutOfRangeException("width", "Width can not be less than or equal to zero.");
            }

            if (height <= 0)
            {
                throw new ArgumentOutOfRangeException("height", "Height can not be less than or equal to zero.");
            }
            
            if (Directory.Exists(this.path))
            {
                return new SVGGraphicsAdapter(string.Format("{0}/{1}.svg", this.path, dependency.Title), width, 
                                              height);
            }
            else
            {
                return new SVGGraphicsAdapter(this.path, width, height);
            }
        }

    }

}
