/*
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDeveloper
 */

using System;
using System.IO;
using NUnit.Framework;

namespace TopCoder.Rules.DependencyMap.View
{

    /// <summary>
    /// Unit test fixture for FixedGridLayoutManager.
    ///
    /// Testing of FixedGridLayoutManager includes the accuracy testing of the layout algorithm,
    /// to verify that each parameter works independently.
    /// </summary>
    [TestFixture]
    public class FixedGridLayoutManagerUnitTests
    {

        /// <summary>
        /// The path to persist to.
        /// </summary>
        private string path = @"../../test_files/unittests";

        /// <summary>
        /// The filename to persist to.
        /// </summary>
        private string filename = @"unittest-";

        /// <summary>
        /// FixedGridLayoutManager instance to test.
        /// </summary>
        private FixedGridLayoutManager manager = null;

        /// <summary>
        /// ShapeFactory instance to use.
        /// </summary>
        private ShapeFactory factory = null;

        /// <summary>
        /// Graphics Renderer to use.
        /// </summary>
        private SVGRenderer renderer = null;

        /// <summary>
        /// Create new layout manager, shape factory and renderer renderer for each test.
        /// </summary>
        [SetUp]
        protected void SetUp()
        {
            manager = new FixedGridLayoutManager();
            factory = new ShapeFactory();
            renderer = new SVGRenderer(path);
        }

        /// <summary>
        /// Remove the file and dispose the renderer renderer.
        /// </summary>
        [TearDown]
        protected void TearDown()
        {
        }

        /// <summary>
        /// Create instance.
        /// </summary>
        [Test]
        public void Create()
        {
            manager = new FixedGridLayoutManager();
        }

        /// <summary>
        /// Create instance with colPadding and rowPadding.
        /// </summary>
        [Test]
        public void Create_ColPaddingRowPadding()
        {
            manager = new FixedGridLayoutManager(10, 10);
        }

        /// <summary>
        /// Create instance with negative colPadding.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void Create_NegativeColPadding()
        {
            new FixedGridLayoutManager(-1, 10);
        }

        /// <summary>
        /// Create instance with negative rowPadding.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void Create_NegativeRowPadding()
        {
            new FixedGridLayoutManager(10, -1);
        }

        /// <summary>
        /// Create instance with colPadding, rowPadding and textMargin.
        /// </summary>
        [Test]
        public void Create_ColPaddingRowPaddingTextMargin()
        {
            manager = new FixedGridLayoutManager(10, 10, 10);
        }

        /// <summary>
        /// Create instance with negative colPadding.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void Create_NegativeColPadding2()
        {
            new FixedGridLayoutManager(-1, 10, 10);
        }

        /// <summary>
        /// Create instance with negative rowPadding.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void Create_NegativeRowPadding2()
        {
            new FixedGridLayoutManager(10, -1, 10);
        }

        /// <summary>
        /// Create instance with negative textMargin.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void Create_NegativeTextMargin()
        {
            new FixedGridLayoutManager(10, 10, -1);
        }

        /// <summary>
        /// Layout single dependency.
        /// </summary>
        [Test]
        public void Layout_SingleDependency()
        {
            Dependency dependency = new Dependency(filename + "single_dependency");
            
            manager.Layout(dependency, factory, renderer);
        }

        /// <summary>
        /// Layout dependency tree.
        /// </summary>
        [Test]
        public void Layout_DependencyTree()
        {
            Dependency dependency = new Dependency(filename + "dependency_tree");
            dependency.AddDependant(new Dependency("SVG Generator"));
            dependency.AddDependant(new Dependency("Base Exception"));

            manager.Layout(dependency, factory, renderer);
        }

        /// <summary>
        /// Layout seen dependency.
        /// </summary>
        [Test]
        public void Layout_SeenDependency()
        {
            Dependency dependency = new Dependency(filename + "seen_dependency");
            dependency.AddDependant(dependency);

            manager.Layout(dependency, factory, renderer);
        }

        /// <summary>
        /// Layout shapeOverride.
        /// </summary>
        [Test]
        public void Layout_ShapeOverride()
        {
            Dependency dependency = new Dependency(filename + "shape_override", ShapeFactory.SeenElement);

            manager.Layout(dependency, factory, renderer);
        }

        /// <summary>
        /// Layout colPadding and rowPadding.
        /// </summary>
        [Test]
        public void Layout_ColPaddingRowPadding()
        {
            manager = new FixedGridLayoutManager(0, 50);
            Dependency dependency = new Dependency(filename + "colpadding_rowpadding");
            dependency.AddDependant(new Dependency("SVG Generator"));
            dependency.AddDependant(new Dependency("Base Exception"));

            manager.Layout(dependency, factory, renderer);
        }

        /// <summary>
        /// Layout textMargin.
        /// </summary>
        [Test]
        public void Layout_TextMargin()
        {
            manager = new FixedGridLayoutManager(10, 10, 50);
            Dependency dependency = new Dependency(filename + "text_margin");
            dependency.AddDependant(new Dependency("SVG Generator"));
            dependency.AddDependant(new Dependency("Base Exception"));
            
            manager.Layout(dependency, factory, renderer);
        }

        /// <summary>
        /// Layout testing rule 1 of combining.
        /// </summary>
        [Test]
        public void Layout_CombineRule1()
        {
            manager = new FixedGridLayoutManager(0, 50);
            Dependency dependency = new Dependency(filename + "combine_rule1");
            Dependency b = new Dependency("B");
            Dependency c = new Dependency("C");
            Dependency d = new Dependency("D");
            Dependency e = new Dependency("E");
            
            dependency.AddDependant(e);
            dependency.AddDependant(b);
            
            b.AddDependant(c);
            b.AddDependant(d);
            
            e.AddDependant(c);
            
            manager.Layout(dependency, factory, renderer);
        }

        /// <summary>
        /// Layout testing rule 2 of combining.
        /// </summary>
        [Test]
        public void Layout_CombineRule2()
        {
            manager = new FixedGridLayoutManager(0, 10);
            Dependency dependency = new Dependency(filename + "combine_rule2");
            Dependency b = new Dependency("B");
            Dependency c = new Dependency("C");
            Dependency d = new Dependency("D");
            Dependency e = new Dependency("E");
            Dependency f = new Dependency("F");
            Dependency g = new Dependency("G");
            Dependency h = new Dependency("H");
            
            dependency.AddDependant(b);
            dependency.AddDependant(e);
            dependency.AddDependant(g);
            
            b.AddDependant(c);
            b.AddDependant(d);
            
            e.AddDependant(f);
            
            g.AddDependant(h);
           	g.AddDependant(d);
            
            manager.Layout(dependency, factory, renderer);
        }

        /// <summary>
        /// Layout testing rule 3a of combining.
        /// </summary>
        [Test]
        public void Layout_CombineRule3a()
        {
            manager = new FixedGridLayoutManager(0, 50);
            Dependency dependency = new Dependency(filename + "combine_rule3a");
            Dependency b = new Dependency("B");
            Dependency c = new Dependency("C");
            Dependency d = new Dependency("D");
            Dependency e = new Dependency("E");
            Dependency f = new Dependency("F");
            Dependency g = new Dependency("G");
            
            dependency.AddDependant(b);
            dependency.AddDependant(g);
            
            b.AddDependant(c);
            b.AddDependant(d);
            
            g.AddDependant(c);
            g.AddDependant(d);
            
            d.AddDependant(e);
            d.AddDependant(f);
            
            manager.Layout(dependency, factory, renderer);
        }

        /// <summary>
        /// Layout testing rule 3b of combining.
        /// </summary>
        [Test]
        public void Layout_CombineRule3b()
        {
            manager = new FixedGridLayoutManager(0, 50);
            Dependency dependency = new Dependency(filename + "combine_rule3b");
            Dependency b = new Dependency("B");
            Dependency c = new Dependency("C");
            Dependency d = new Dependency("D");
            Dependency e = new Dependency("E");
            Dependency f = new Dependency("F");
            Dependency g = new Dependency("G");
            Dependency h = new Dependency("H");
            Dependency i = new Dependency("I");
            
            dependency.AddDependant(b);
            dependency.AddDependant(e);
            dependency.AddDependant(g);
            
            b.AddDependant(c);
            b.AddDependant(d);
            
            e.AddDependant(c);
            e.AddDependant(d);
            e.AddDependant(f);
            
            g.AddDependant(d);
            g.AddDependant(h);
            g.AddDependant(i);
            
            manager.Layout(dependency, factory, renderer);
        }

        /// <summary>
        /// Layout testing rule 3a of combining.
        /// </summary>
        [Test]
        public void Layout_BlankFigure2()
        {
            manager = new FixedGridLayoutManager(0, 50);
            Dependency dependency = new Dependency(filename + "blank_figure2");
            Dependency b = new Dependency("B");
            Dependency c = new Dependency("C");
            Dependency d = new Dependency("D");
            Dependency e = new Dependency("E");
            Dependency f = new Dependency("F");
            Dependency g = new Dependency("G");
            Dependency h = new Dependency("H");
            Dependency i = new Dependency("I");
            Dependency j = new Dependency("J");
            Dependency k = new Dependency("K");
            
            dependency.AddDependant(b);
            
            b.AddDependant(c);
            b.AddDependant(d);
            b.AddDependant(k);
            
            d.AddDependant(e);
            d.AddDependant(h);
            d.AddDependant(i);
            d.AddDependant(j);

            e.AddDependant(f);
            e.AddDependant(h);
            
            manager.Layout(dependency, factory, renderer);
        }

        /// <summary>
        /// Test Layout() with null dependency.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void Layout_NullDependency()
        {
            manager.Layout(null, factory, renderer);
        }

        /// <summary>
        /// Test Layout() with null shape factory.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void Layout_NullShapeFactory()
        {
            manager.Layout(new Dependency("a"), null, renderer);
        }

        /// <summary>
        /// Test Layout() with null renderer renderer.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void Layout_NullRendererGraphics()
        {
            manager.Layout(new Dependency("a"), factory, null);
        }
    }

}
