/*
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDeveloper
 */


namespace TopCoder.Rules.DependencyMap.View
{
	using System;
	using System.IO;
	using NUnit.Framework;
	using System.Drawing;
	using System.Drawing.Drawing2D;
	using System.Drawing.Imaging;

    /// <summary>
    /// Unit test fixture for GraphicsAdapter.
    ///
    /// Testing of GraphicsAdpater verifies the adapter interacts properly with the .NET Graphics interface.
    /// </summary>
    [TestFixture]
    public class GraphicsAdapterUnitTests
    {
        /// <summary>
        /// The font to use.
        /// </summary>
        private Font font = new Font(FontFamily.GenericSansSerif, 12);

        /// <summary>
        /// The GraphicsAdapter instance to test.
        /// </summary>
        private GraphicsAdapter adapter = null;

        /// <summary>
        /// Thie Bitmap that the operations are performed on.
        /// </summary>
        private Bitmap bitmap = null;

        /// <summary>
        /// Create a new adapter for each test.
        /// </summary>
        [SetUp]
        protected void SetUp()
        {
        	bitmap = new Bitmap(800, 600, PixelFormat.Format24bppRgb);
        	adapter = new GraphicsAdapter(Graphics.FromImage(bitmap));
        }

        /// <summary>
        /// Remove the file and dispose the adapter.
        /// </summary>
        [TearDown]
        protected void TearDown()
        {
            if (adapter != null) {
                adapter.Dispose();
            }
        }

        /// <summary>
        /// Match the output against predefined sample.
        /// </summary>
        /// <param name="sampleFile">the sample name</param>
        private void Match(string sampleFile)
        {
            sampleFile = string.Format(@"../../test_files/samples/{0}.bmp", sampleFile);

            try
            {
   /*         	BitmapData bits = bitmap.LockBits(Rectangle.Empty, ImageLockMode.ReadOnly, PixelFormat.Format24bppRgb);

            	// TODO: Do this.            	
               using (StreamReader output = new StreamReader(new MemoryStream(bits)))
                {
                    using (StreamReader sample = new StreamReader(sampleFile))
                    {
                        Assert.AreEqual(sample.ReadToEnd(), output.ReadToEnd(), "Output mismatches.");
                    }
                }*/
            }

            catch (Exception)
            {
                Assert.Fail("Failed to compare files.");
            }
        }

        /// <summary>
        /// Create instance with Graphics parameter.
        /// </summary>
        [Test]
        public void Create_Graphics()
        {
            adapter.Dispose();

        	Graphics graphics = Graphics.FromImage(new Bitmap(800, 600, PixelFormat.Format24bppRgb));

        	adapter = new GraphicsAdapter(graphics);
        }

        /// <summary>
        /// Create instance with null Graphics parameter.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void Create_NullGraphics()
        {
            new GraphicsAdapter(null);
        }
       
        /// <summary>
        /// Test DrawPath().
        /// </summary>
        [Test]
        public void DrawPath()
        {
            adapter.DrawPath(new Pen(Color.Black), new ShapeFactory()[ShapeFactory.RelationshipArrow]);
            adapter.Complete();
            Match("raw_arrow");
        }

        /// <summary>
        /// Test DrawPath() with null pen.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void DrawPath_NullPen()
        {
            adapter.DrawPath(null, new ShapeFactory()[ShapeFactory.RelationshipArrow]);
        }

        /// <summary>
        /// Test DrawPath() with null path.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void DrawPath_NullPath()
        {
            adapter.DrawPath(new Pen(Color.Black), null);
        }

        /// <summary>
        /// Test DrawString().
        /// </summary>
        [Test]
        public void DrawString()
        {
            adapter.DrawString("Dependency Map Renderer", font, new SolidBrush(Color.Black), new PointF(15, 23));
            adapter.Complete();
            Match("string");
        }

        /// <summary>
        /// Test DrawString() with null text.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void DrawString_NullText()
        {
            adapter.DrawString(null, font, new SolidBrush(Color.Black), new PointF(15, 23));
        }

        /// <summary>
        /// Test DrawString() with null font.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void DrawString_NullFont()
        {
            adapter.DrawString("Dependency Map Renderer", null, new SolidBrush(Color.Black), new PointF(15, 23));
        }

        /// <summary>
        /// Test DrawString() with null brush.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void DrawString_NullBrush()
        {
            adapter.DrawString("Dependency Map Renderer", font, null, new PointF(15, 23));
        }

        /// <summary>
        /// Test MeasreString().
        /// </summary>
        [Test]
        public void MeasureString()
        {
            string text = "Dependency Map Renderer";
            SizeF size = Graphics.FromImage(new Bitmap(1, 1)).MeasureString(text, font);
            Assert.AreEqual(size, adapter.MeasureString(text, font), "Incorrect measurement.");
        }

        /// <summary>
        /// Test MeasureString() with null text.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void MeasureString_NullText()
        {
            adapter.MeasureString(null, font);
        }

        /// <summary>
        /// Test MeasureString() with null font.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void MeasureString_NullFont()
        {
            adapter.MeasureString("text", null);
        }

        /// <summary>
        /// Test Complete().
        /// </summary>
        [Test]
        public void Complete()
        {
            adapter.Complete();
        }

        /// <summary>
        /// Test Dispose().
        /// </summary>
        [Test]
        public void Dispose()
        {
            adapter.Dispose();
            adapter = null;
        }

    }

}
