/*
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDeveloper
 */

using System;
using System.IO;
using NUnit.Framework;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace TopCoder.Rules.DependencyMap.View
{

    /// <summary>
    /// Unit test fixture for SVGGraphicsAdapter.
    ///
    /// Testing of SVGGraphicsAdpater verifies the adapter interacts properly with the SVG component.
    /// </summary>
    [TestFixture]
    public class SVGGraphicsAdapterUnitTests
    {

        /// <summary>
        /// The font to use.
        /// </summary>
        private Font font = new Font(FontFamily.GenericSansSerif, 12);

        /// <summary>
        /// The filename to persist to.
        /// </summary>
        private string filename = @"../../test_files/unittest.svg";

        /// <summary>
        /// The SVGGraphicsAdapter instance to test.
        /// </summary>
        private SVGGraphicsAdapter adapter = null;

        /// <summary>
        /// Create a new adapter for each test.
        /// </summary>
        [SetUp]
        protected void SetUp()
        {
            adapter = new SVGGraphicsAdapter(filename, 1200, 800);
        }

        /// <summary>
        /// Remove the file and dispose the adapter.
        /// </summary>
        [TearDown]
        protected void TearDown()
        {
            if (adapter != null) {
                adapter.Dispose();
            }
            File.Delete(filename);
        }

        /// <summary>
        /// Match the output against predefined sample.
        /// </summary>
        /// <param name="sampleFile">the sample name</param>
        private void Match(string sampleFile)
        {
            sampleFile = string.Format(@"../../test_files/samples/{0}.svg", sampleFile);

            try
            {
                using (StreamReader output = new StreamReader(filename))
                {
                    using (StreamReader sample = new StreamReader(sampleFile))
                    {
                        Assert.AreEqual(sample.ReadToEnd(), output.ReadToEnd(), "Output mismatches.");
                    }
                }
            }

            catch (Exception)
            {
                Assert.Fail("Failed to compare files.");
            }
        }

        /// <summary>
        /// Create instance with filename.
        /// </summary>
        [Test]
        public void Create_Filename()
        {
            adapter.Dispose();

            adapter = new SVGGraphicsAdapter(filename, 1, 1);
        }

        /// <summary>
        /// Create instance with null filename.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void Create_NullFilename()
        {
            new SVGGraphicsAdapter(null, 1, 1);
        }

        /// <summary>
        /// Create instance with empty filename.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void Create_EmptyFilename()
        {
            new SVGGraphicsAdapter(" ", 1, 1);
        }

        /// <summary>
        /// Create instance with zero width.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void Create_ZeroWidth()
        {
            new SVGGraphicsAdapter("Test", 0, 1);
        }

        /// <summary>
        /// Create instance with zero height.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void Create_ZeroHeight()
        {
            new SVGGraphicsAdapter("Test", 1, 0);
        }

        /// <summary>
        /// Create instance with negative width.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void Create_NegativeWidth()
        {
            new SVGGraphicsAdapter("Test", -1, 1);
        }

        /// <summary>
        /// Create instance with negative height.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void Create_NegativeHeight()
        {
            new SVGGraphicsAdapter("Test", 1, -1);
        }
        
        /// <summary>
        /// Test DrawPath().
        /// </summary>
        [Test]
        public void DrawPath()
        {
            adapter.DrawPath(new Pen(Color.Black), new ShapeFactory()[ShapeFactory.RelationshipArrow]);
            adapter.Complete();
            Match("raw_arrow");
        }

        /// <summary>
        /// Test DrawPath() with null pen.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void DrawPath_NullPen()
        {
            adapter.DrawPath(null, new ShapeFactory()[ShapeFactory.RelationshipArrow]);
        }

        /// <summary>
        /// Test DrawPath() with null path.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void DrawPath_NullPath()
        {
            adapter.DrawPath(new Pen(Color.Black), null);
        }

        /// <summary>
        /// Test DrawString().
        /// </summary>
        [Test]
        public void DrawString()
        {
            adapter.DrawString("Dependency Map Renderer", font, new SolidBrush(Color.Black), new PointF(15, 23));
            adapter.Complete();
            Match("string");
        }

        /// <summary>
        /// Test DrawString() with null text.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void DrawString_NullText()
        {
            adapter.DrawString(null, font, new SolidBrush(Color.Black), new PointF(15, 23));
        }

        /// <summary>
        /// Test DrawString() with null font.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void DrawString_NullFont()
        {
            adapter.DrawString("Dependency Map Renderer", null, new SolidBrush(Color.Black), new PointF(15, 23));
        }

        /// <summary>
        /// Test DrawString() with null brush.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void DrawString_NullBrush()
        {
            adapter.DrawString("Dependency Map Renderer", font, null, new PointF(15, 23));
        }

        /// <summary>
        /// Test MeasreString().
        /// </summary>
        [Test]
        public void MeasureString()
        {
            string text = "Dependency Map Renderer";
            SizeF size = Graphics.FromImage(new Bitmap(1, 1)).MeasureString(text, font);
            Assert.AreEqual(size, adapter.MeasureString(text, font), "Incorrect measurement.");
        }

        /// <summary>
        /// Test MeasureString() with null text.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void MeasureString_NullText()
        {
            adapter.MeasureString(null, font);
        }

        /// <summary>
        /// Test MeasureString() with null font.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void MeasureString_NullFont()
        {
            adapter.MeasureString("text", null);
        }

        /// <summary>
        /// Test Complete().
        /// </summary>
        [Test]
        public void Complete()
        {
            adapter.Complete();
            Assert.IsTrue(File.Exists(filename), "Output does not exist.");
        }

        /// <summary>
        /// Test Dispose().
        /// </summary>
        [Test]
        public void Dispose()
        {
            adapter.Dispose();
            adapter = null;
        }

    }

}
