/*
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDeveloper
 */

using System;
using System.IO;
using NUnit.Framework;

namespace TopCoder.Rules.DependencyMap.View
{

    /// <summary>
    /// Unit test fixture for SVGRenderer.
    ///
    /// Testing of SVGRenderer verifies it can work separately for directory or file.
    /// </summary>
    [TestFixture]
    public class SVGRendererUnitTests
    {

        /// <summary>
        /// The directory to use.
        /// </summary>
        private string directory = @"../../test_files";

        /// <summary>
        /// The file to use.
        /// </summary>
        private string file = @"../../test_files/unittest.svg";

        /// <summary>
        /// The dependency to use.
        /// </summary>
        private Dependency dependency = new Dependency("unittest");

        /// <summary>
        /// The SVGRenderer instance to test.
        /// </summary>
        private SVGRenderer renderer = null;

        /// <summary>
        /// Create a new renderer for each test.
        /// </summary>
        [SetUp]
        protected void SetUp()
        {
            renderer = new SVGRenderer(file);
        }

        /// <summary>
        /// Create instance with path.
        /// </summary>
        [Test]
        public void Create_Path()
        {
            renderer = new SVGRenderer(file);
            Assert.AreEqual(file, renderer.Path, "Path mismatches.");
        }

        /// <summary>
        /// Create instance with null path.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void Create_NullPath()
        {
            new SVGRenderer(null);
        }

        /// <summary>
        /// Test getter Path.
        /// </summary>
        [Test]
        public void GetterPath()
        {
            Assert.AreEqual(file, renderer.Path, "Path mismatches.");
        }

        /// <summary>
        /// Test setter Path.
        /// </summary>
        [Test]
        public void SetterPath()
        {
            renderer.Path = directory;
            Assert.AreEqual(directory, renderer.Path, "Path mismatches.");
        }

        /// <summary>
        /// Test setter Path with null path.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void SetterPath_NullPath()
        {
            renderer.Path = null;
        }

        /// <summary>
        /// Test GetGraphics() with file path.
        /// </summary>
        [Test]
        public void GetGraphics_FilePath()
        {
            renderer.Path = file;
            IGraphics graphics = renderer.GetGraphics(dependency, 1, 1);
            graphics.Complete();
            graphics.Dispose();

            Assert.IsTrue(File.Exists(file), "Output does not exist.");
            File.Delete(file);
        }

        /// <summary>
        /// Test GetGraphics() with directory path.
        /// </summary>
        [Test]
        public void GetGraphics_DirectoryPath()
        {
            renderer.Path = directory;
            IGraphics graphics = renderer.GetGraphics(dependency, 1, 1);
            graphics.Complete();
            graphics.Dispose();

            Assert.IsTrue(File.Exists(file), "Output does not exist.");
            File.Delete(file);
        }

        /// <summary>
        /// Test GetGraphics() with null dependency.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void GetGraphics_NullDependency()
        {
            renderer.GetGraphics(null, 1, 1);
        }

        /// <summary>
        /// Test GetGraphics() with a zero width
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void GetGraphics_ZeroWidth()
        {
            renderer.GetGraphics(dependency, 0, 1);
        }

        /// <summary>
        /// Test GetGraphics() with a zero height
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void GetGraphics_ZeroHeight()
        {
            renderer.GetGraphics(dependency, 1, 0);
        }

        /// <summary>
        /// Test GetGraphics() with a negative width
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void GetGraphics_NegativeWidth()
        {
            renderer.GetGraphics(dependency, -5, 1);
        }

        /// <summary>
        /// Test GetGraphics() with a negative height
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void GetGraphics_NegativeHeight()
        {
            renderer.GetGraphics(dependency, 1, -5);
        }
        
    }

}
