/**
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights Reserved.
 */
package com.topcoder.rules.dependencymap.view;

import java.io.File;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.geom.GeneralPath;
import com.topcoder.util.image.svg2dgraphics.SVGGraphics2D;

/**
 * This class is an Adapter class between the RendererGraphics interface and the SVG Generator component
 * (specifically the SVGGraphics2D class). This class attempts to simplify the SVG component and offer only those
 * features the RendererGraphics interface requires. This class is created by the SVGRenderer in response to the
 * GetGraphics call and will simply proxy its call to the embedded SVGGraphics2D class.
 *
 * Note: the SVGRenderer should never reuse this class because there is no way to reset the SVG information
 * from an older drawing.
 *
 * @author Pops
 * @author TCSDeveloper
 */
class SVGGraphicsAdapter implements RendererGraphics {

    /**
     * This represents the underlying SVGGraphics2D that this adapter class uses. This variable is created at
     * object instantiation time, is immutable (the reference, not the contents) and is accessed by all methods.
     * This variable has been marked final to ensure it's immutability.
     */
    private final SVGGraphics2D g2d = new SVGGraphics2D();

    /**
     * This variable represents the file name that the diagram will be rendered to. The variable is initially
     * set in the constructor, is immutable and will never be null. The variable is accessed in the Complete
     * method. This variable has been marked final to ensure it's immutability.
     */
    private final String filename;

    /**
     * Constructors the adapter class. Simple saves the argument to the like named variable.
     *
     * @param filename the filename to save to
     * @throws NullPointerException if filename is null
     * @throws IllegalArgumentException if filename (trim'd) is null
     */
    SVGGraphicsAdapter(String filename) {
        if (filename == null) {
            throw new NullPointerException("filename should not be null.");
        }
        if (filename.trim().length() == 0) {
            throw new IllegalArgumentException("filename should not be empty.");
        }
        this.filename = filename;
    }

    /**
     * Get's the current color. This method simply returns g2d.getColor().
     *
     * @return The current color
     */
    public Color getColor() {
        return this.g2d.getColor();
    }

    /**
     * Set's the current color.  This method simply returns g2d.setColor(color).
     *
     * @param color The color to use
     * @throws NullPointerException if color is null
     */
    public void setColor(Color color) {
        if (color == null) {
            throw new NullPointerException("color should not be null.");
        }
        this.g2d.setColor(color);
    }

    /**
     * Get's the current font. This method simply returns g2d.getFont().
     *
     * @return The current font
     */
    public Font getFont() {
        return this.g2d.getFont();
    }

    /**
     * Set's the current font. This method simply returns g2d.setFont(font).
     *
     * @param font The font to use
     * @throws NullPointerException if font is null
     */
    public void setFont(Font font) {
        if (font == null) {
            throw new NullPointerException("font should not be null.");
        }
        this.g2d.setFont(font);
    }

    /**
     * Get's the font's metrics. This method simply returns g2d.getFontMetrics(font).
     *
     * @param font The font to get metrics for
     * @return The font metrics
     * @throws NullPointerException if font is null
     */
    public FontMetrics getFontMetrics(Font font) {
        if (font == null) {
            throw new NullPointerException("font should not be null.");
        }
        return this.g2d.getFontMetrics(font);
    }

    /**
     * Draws the specified shape. Simply calls g2d.draw(shape).
     *
     * @param shape The shape to draw
     * @throws NullPointerException if shape is null
     */
    public void draw(GeneralPath shape) {
        if (shape == null) {
            throw new NullPointerException("shape should not be null.");
        }
        this.g2d.draw(shape);
    }

    /**
     * Draws the specified string at the specified position. Simply calls g2d.drawString(str, x, y).
     *
     * @param str The text to draw
     * @param x The horizontal position to draw the text at
     * @param y The vertical position (baseline) to draw the text at
     * @throws NullPointerException if str is null
     */
    public void drawString(String str, int x, int y) {
        if (str == null) {
            throw new NullPointerException("str should not be null.");
        }
        this.g2d.drawString(str, x, y);
    }

    /**
     * Completes the processing by saving the file. This method simply calls g2d.renderSVGImage(filename).
     * If any exception is thrown, catch/wrap/throw a RendererException with the exception embedded.
     *
     * @throws RendererException if an exception occurs completing processing
     */
    public void complete() throws RendererException {
        try {
            this.g2d.renderSVGImage(new File(filename));
        } catch (Exception exception) {
            throw new RendererException("SVG rendering fails.", exception);
        }
    }

    /**
     * Disposes of any held resources. Simply calls g2d.dispose().
     */
    public void dispose() {
        this.g2d.dispose();
    }

}
